@@ -180,7 +180,7 @@ public class IconView : Gtk.Layout {
 		this.selection = new SelectionCollection (collection);
 		
 		collection.Changed += HandleChanged;
-		collection.ItemChanged += HandleItemChanged;
+		collection.ItemsChanged += HandleItemsChanged;
 
 		selection.DetailedChanged += HandleSelectionChanged;
 	}
@@ -201,10 +201,12 @@ public class IconView : Gtk.Layout {
 		QueueResize ();
 	}
 	
-	private void HandleItemChanged (FSpot.IBrowsableCollection sender, int item)
+	private void HandleItemsChanged (FSpot.IBrowsableCollection sender, BrowsableArgs args)
 	{
-		UpdateThumbnail (item);
-		InvalidateCell (item);
+		foreach (int item in args.Items) {
+			UpdateThumbnail (item);
+			InvalidateCell (item);
+		}
 	}
 
 	//
@@ -240,7 +242,7 @@ public class IconView : Gtk.Layout {
 			this.selected_cells = new Hashtable ();
 			this.parent = collection;
 			this.parent.Changed += HandleParentChanged;
-			this.parent.ItemChanged += HandleParentItemChanged;
+			this.parent.ItemsChanged += HandleParentItemsChanged;
 		}
 
 		private void HandleParentChanged (IBrowsableCollection collection)
@@ -269,12 +271,26 @@ public class IconView : Gtk.Layout {
 
 		}
 
-		private void HandleParentItemChanged (IBrowsableCollection collection, int parent_index)
-		{
-			int local_index = this.IndexOf (parent_index);
-			//System.Console.WriteLine ("parent = {0} local = {1}", parent_index, local_index);
-			if (local_index >= 0 && this.ItemChanged != null)
-				this.ItemChanged (collection, local_index);
+		private void HandleParentItemsChanged (IBrowsableCollection collection, BrowsableArgs args)
+		{			
+			if (this.ItemsChanged == null)
+				return;
+
+			ArrayList local_ids = new ArrayList ();
+			foreach (int parent_index in args.Items) {
+				// If the item isn't part of the selection ignore it
+				if (!this.Contains (collection [parent_index]))
+					return;
+
+				int local_index = this.IndexOf (parent_index);
+				if (local_index >= 0)
+					local_ids.Add (local_index);
+			}
+
+			if (local_ids.Count == 0)
+				return;
+
+			ItemsChanged (this, new BrowsableArgs ((int [])local_ids.ToArray (typeof (int))));
 		}
 
 		public int [] Ids {
@@ -389,6 +405,26 @@ public class IconView : Gtk.Layout {
 			
 			SignalChange (ids);
 		}
+		
+		// Remove a range, except the start entry
+		public void Remove (int start, int end)
+		{
+			if (start == -1 || end == -1)
+				return;
+			
+			int current = Math.Min (start + 1, end);
+			int final = Math.Max (start - 1, end);				
+			int count = final - current + 1;
+			int [] ids = new int [count];
+
+			for (int i = 0; i < count; i++) {
+				this.Remove (current, false);
+				ids [i] = current;
+				current++;
+			}
+
+			SignalChange (ids);
+		}
 
 		public int IndexOf (int parent_index)
 		{
@@ -405,22 +441,37 @@ public class IconView : Gtk.Layout {
 		}
 
 		public void Remove (int cell)
+		{
+			Remove (cell, true);
+		}
+
+		private void Remove (int cell, bool notify)
 		{
 			IBrowsableItem item = parent [cell];
 			if (item != null)
-				this.Remove (item);
+				this.Remove (item, notify);
 
 		}
-		
+
 		public void Remove (IBrowsableItem item)
 		{
+			Remove (item, true);
+		}
+
+		private void Remove (IBrowsableItem item, bool notify)
+		{
+			if (item == null)
+				return;
+
 			int parent_index = (int) selected_cells [item];
 			selected_cells.Remove (item);
-			SignalChange (new int [] {parent_index});
+
+			if (notify)
+				SignalChange (new int [] {parent_index});
 		}
 
 		public event IBrowsableCollectionChangedHandler Changed;
-		public event IBrowsableCollectionItemChangedHandler ItemChanged;
+		public event IBrowsableCollectionItemsChangedHandler ItemsChanged;
 
 		public delegate void DetailedCollectionChanged (IBrowsableCollection collection, int [] ids);
 		public event DetailedCollectionChanged DetailedChanged;
@@ -444,15 +495,6 @@ public class IconView : Gtk.Layout {
 				DetailedChanged (this, ids);
 		}
 	}
-	
-	public int CurrentIdx {
-		get {
-			if (selection.Count == 1 && selection.Contains (FocusCell))
-				return FocusCell;
-			else 
-				return -1;
-		}
-	}
 
 	// Updating.
 	public void UpdateThumbnail (int thumbnail_num)
@@ -463,9 +505,7 @@ public class IconView : Gtk.Layout {
 		InvalidateCell (thumbnail_num);
 	}
 
-
 	// Cell Geometry
-
 	public int CellAtPosition (int x, int y)
 	{
 		return CellAtPosition (x, y, true);
@@ -498,7 +538,8 @@ public class IconView : Gtk.Layout {
 	
 	public int TopLeftVisibleCell ()
 	{
-		return CellAtPosition(BORDER_SIZE, (int)Vadjustment.Value + BORDER_SIZE + 8);
+		//return CellAtPosition(BORDER_SIZE, (int)Vadjustment.Value + BORDER_SIZE + 8);
+		return CellAtPosition(BORDER_SIZE, (int) (Vadjustment.Value + Allocation.Height * (Vadjustment.Value / Vadjustment.Upper)) + BORDER_SIZE + 8);
 	}
 
 	public void GetCellCenter (int cell_num, out int x, out int y)
@@ -627,15 +668,6 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
-	static Gdk.Rectangle Expand (Gdk.Rectangle src, int width)
-	{
-		src.X -= width;
-		src.Y -= width;
-		src.Width += width * 2;
-		src.Height += width * 2;
-		return src;
-	}
-
 	int ThrobExpansion (int cell, bool selected)
 	{
 		int expansion = 0;
@@ -681,7 +713,8 @@ public class IconView : Gtk.Layout {
 				    bounds.X, bounds.Y,
 				    bounds.Width - 1, bounds.Height - 1);
 		
-		Gdk.Rectangle focus = Expand (bounds, -3);
+		Gdk.Rectangle focus = Gdk.Rectangle.Inflate (bounds, -3, -3);
+
 		if (HasFocus && thumbnail_num == FocusCell) {
 			Style.PaintFocus(Style, BinWindow, 
 					 cell_state, area, 
@@ -691,14 +724,14 @@ public class IconView : Gtk.Layout {
 		}
 
 		Gdk.Rectangle region = Gdk.Rectangle.Zero;
-		Gdk.Rectangle image_bounds = Expand (bounds, - cell_border_width);
+		Gdk.Rectangle image_bounds = Gdk.Rectangle.Inflate (bounds, -cell_border_width, -cell_border_width);
 		int expansion = ThrobExpansion (thumbnail_num, selected);
 
 		Gdk.Pixbuf thumbnail = null;
 		if (entry != null)
 			thumbnail = entry.ShallowCopyPixbuf ();
 
-		if (Expand (image_bounds, expansion + 1).Intersect (area, out image_bounds) && thumbnail != null) {
+		if (Gdk.Rectangle.Inflate (image_bounds, expansion + 1, expansion + 1).Intersect (area, out image_bounds) && thumbnail != null) {
 			
 			PixbufUtils.Fit (thumbnail, ThumbnailWidth, ThumbnailHeight, 
 					 true, out region.Width, out region.Height);
@@ -709,7 +742,7 @@ public class IconView : Gtk.Layout {
 			if (region.Width != thumbnail.Width && region.Height != thumbnail.Height)
 				cache.Reload (entry, thumbnail_num, thumbnail.Width, thumbnail.Height);
 
-			region = Expand (region, expansion);
+			region = Gdk.Rectangle.Inflate (region, expansion, expansion);
 			Pixbuf temp_thumbnail;			
 			region.Width = System.Math.Max (1, region.Width);
 			region.Height = System.Math.Max (1, region.Height); 
@@ -747,7 +780,7 @@ public class IconView : Gtk.Layout {
 			region.Width = temp_thumbnail.Width;
 			region.Height = temp_thumbnail.Height;
 			
-			Gdk.Rectangle draw = Expand (region, 1);
+			Gdk.Rectangle draw = Gdk.Rectangle.Inflate (region, 1, 1);
 			
 			if (!temp_thumbnail.HasAlpha) 
 				Style.PaintShadow (Style, BinWindow, cell_state,
@@ -968,10 +1001,7 @@ public class IconView : Gtk.Layout {
 		offscreen.Subtract (onscreen);
 
 		PreloadRegion (offscreen, ystep);
-		offscreen.Destroy ();
-		//PreloadRegion (onscreen, ystep);
 		Preload (area, false);
-		onscreen.Destroy ();
 
 		y_offset = (int) Vadjustment.Value;
 		x_offset = (int) Hadjustment.Value;
@@ -1079,7 +1109,12 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
-	public void ScrollTo (int cell_num)
+ 	public void ScrollTo (int cell_num)
+ 	{
+		ScrollTo (cell_num, true);
+	}
+
+	public void ScrollTo (int cell_num, bool center)
 	{
 		if (!IsRealized)
 			return;
@@ -1093,7 +1128,10 @@ public class IconView : Gtk.Layout {
 		if (y + cell_height > adjustment.Upper)
 			UpdateLayout ();
 
-		adjustment.Value = y + cell_height / 2 - adjustment.PageSize / 2;
+		if (center)
+			adjustment.Value = y + cell_height / 2 - adjustment.PageSize / 2;
+		else
+			adjustment.Value = y;
 		
 		adjustment.ChangeValue ();
 
@@ -1167,9 +1205,6 @@ public class IconView : Gtk.Layout {
 
 	public void InvalidateCell (int order) 
 	{		
-		if (dead) 
-			return;
-
 		Rectangle cell_area = CellBounds (order);
 		// FIXME where are we computing the bounds incorrectly
 		cell_area.Width -= 1;
@@ -1304,10 +1339,16 @@ public class IconView : Gtk.Layout {
 			FocusCell += cells_per_row;
 			break;
 		case Gdk.Key.Left:
-			FocusCell--;
+			if (control && shift)
+				FocusCell -= FocusCell % cells_per_row;
+			else
+				FocusCell--;
 			break;
 		case Gdk.Key.Right:
-			FocusCell++;
+			if (control && shift)
+				FocusCell += cells_per_row - (FocusCell % cells_per_row) - 1;
+			else
+				FocusCell++;
 			break;
 		case Gdk.Key.Up:
 			FocusCell -= cells_per_row;
@@ -1333,13 +1374,19 @@ public class IconView : Gtk.Layout {
 			return;		
 		}
 		
-		if (FocusCell < 0 || FocusCell > collection.Count - 1) {
-			FocusCell = focus_old;
+		FocusCell = Math.Max (FocusCell, 0);
+		FocusCell = Math.Min (FocusCell, collection.Count - 1);
+		
+		if (FocusCell == focus_old) {
 			args.RetVal = false;
-		}	
+			return;
+		}
 
 		if (shift) {
-			selection.Add (focus_old, FocusCell);
+			if (focus_old != FocusCell && selection.Contains (focus_old) && selection.Contains (FocusCell))
+				selection.Remove (FocusCell, focus_old);
+			else
+				selection.Add (focus_old, FocusCell);
 		} else if (!control) {
 			selection.Clear ();
 			selection.Add (FocusCell);
@@ -1348,7 +1395,6 @@ public class IconView : Gtk.Layout {
 		ScrollTo (FocusCell);
 	}
 
-	bool dead = false;
 	private void HandleDestroyed (object sender, System.EventArgs args)
 	{
 		cache.OnPixbufLoaded -= HandlePixbufLoaded;
