@@ -1,3 +1,4 @@
+using System;
 using System.IO;
 
 namespace FSpot {
@@ -11,9 +12,9 @@ namespace FSpot {
 			this.path = path;
 		}
 		
-		public virtual System.IO.Stream PixbufStream ()
+		public virtual Stream PixbufStream ()
 		{
-			return System.IO.File.OpenRead (this.path);
+			return File.OpenRead (this.path);
 		}
 
 		static ImageFile ()
@@ -29,8 +30,9 @@ namespace FSpot {
 			name_table [".pef"] = typeof (FSpot.Tiff.NefFile);
 			name_table [".raw"] = typeof (FSpot.Tiff.NefFile);
 			name_table [".tiff"] = typeof (FSpot.Tiff.TiffFile);
-			name_table [".orf"] =  typeof (FSpot.Tiff.NefFile);
 			name_table [".tif"] = typeof (FSpot.Tiff.TiffFile);
+			name_table [".orf"] =  typeof (FSpot.Tiff.NefFile);
+			name_table [".srf"] = typeof (FSpot.Tiff.NefFile);
 			name_table [".dng"] = typeof (FSpot.Tiff.DngFile);
 			name_table [".crw"] = typeof (FSpot.Ciff.CiffFile);
 			name_table [".ppm"] = typeof (FSpot.Pnm.PnmFile);
@@ -91,10 +93,24 @@ namespace FSpot {
 			return PixbufOrientation.TopLeft;
 		}
 		
+		// FIXME this need to have an intent just like the loading stuff.
+		public virtual Cms.Profile GetProfile ()
+		{
+			return null;
+		}
+		
 		public virtual System.DateTime Date 
 		{
 			get {
-				return File.GetCreationTimeUtc  (this.path);
+				// FIXME mono uses the file change time (ctime) incorrectly
+				// as the creation time so we try to work around that slightly
+				DateTime create = File.GetCreationTimeUtc  (this.path);
+				DateTime write =  File.GetLastWriteTimeUtc (this.path);
+
+				if (create < write)
+					return create;
+				else 
+					return write;
 			}
 		}
 
